import sys
import os
sys.path.insert(0, os.path.abspath('..'))


import trace_pb2 as trace

from datetime import datetime



class AWSFact:

    def __init__(self, path):
        self.PATH = path

    AWS_LAMBDA_COST = {128: 0.0000002083, 512: 0.0000008333, 1024: 0.0000016667, 1536: 0.0000025000, 2048: 0.0000033333,
                       3008: 0.0000048958}

    PATH = ""

    def get_closes(self, mb):
        if mb <= 128:
            return self.AWS_LAMBDA_COST[128]
        elif mb <= 512:
            return self.AWS_LAMBDA_COST[512]
        elif mb <= 1024:
            return self.AWS_LAMBDA_COST[1024]
        elif mb <= 2048:
            return self.AWS_LAMBDA_COST[2048]
        else:
            return self.AWS_LAMBDA_COST[3008]

    @staticmethod
    def get_instances(self, path=None):
        if path is None:
            return AWSFact("/proc/self/cgroup")
        else:
            return AWSFact(path)

    def collect(self, trace: trace.Trace, context):
        if context is None:
            raise Exception
        trace.Memory = context.memory_limit_in_mb
        trace.Tags["fname"] = context.function_name
        trace.Tags["fver"] = context.function_version
        trace.Tags["rid"] = context.aws_request_id
        trace.Logs[str(datetime.now().timestamp() * 1000)] = "RemainingTime {}".format(
            context.get_remaining_time_in_millis())
        if context.client_context is not None:
            if "inlcudeEnvironment" in Fact.config and Fact.config["inlcudeEnvironment"]:
                trace.Env.update(context.client_context.env)
                trace.Env.update(context.client_context.custom)
        elat = (datetime.now().timestamp() * 1000 - Fact.start_time) // 100
        cost = elat * self.get_closes(context.memory_limit_in_mb)
        trace.Cost = cost

    FREEZER_OFFSET = len("freezer:/sandbox-")

    def read_cgroup_ids(self, trace):
        file = open(self.PATH, 'r')
        lines = file.readlines()
        found = 0
        for line in lines:
            index = line.find("freezer")
            if index > 0:
                trace.Tags["freezer"] = line[index + self.FREEZER_OFFSET:].strip()
                found += 1
            index = line.find("sandbox-root-")
            if index > 0:
                line = line[index:]
                if len(line) < 57:
                    raise ValueError
                host = line[13:19]
                trace.Tags["host"] = host
                trace.HostID = host
                trace.Tags["service"] = line[36:42]
                trace.Tags["sandbox"] = line[51:57]
                found += 1
            if found >= 2:
                break

    def init(self, trace, context=None):
        log_name = os.environ.get("AWS_LAMBDA_LOG_STREAM_NAME")
        trace.Platform = 'AWS'
        trace.ContainerId = log_name
        trace.Region = os.environ.get("AWS_REGION")
        uptime = Fact.readFile("/proc/uptime").trim()
        trace.Tags["uptime"] = uptime
        try:
            self.read_cgroup_ids(trace)
        except Exception:
            trace.putTags("host", "U" + uptime)
            trace.setHostID("U" + uptime)
            trace.putTags("service", "undefined")
            trace.putTags("sandbox", "undefined")
            trace.putTags("freezer", "undefined")
        if context != None:
            self.collect(trace, context)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: trace.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='trace.proto',
  package='factclient',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0btrace.proto\x12\nfactclient\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\xab\x05\n\x05Trace\x12\n\n\x02ID\x18\x01 \x01(\t\x12\x0f\n\x07\x43hildOf\x18\x02 \x01(\t\x12-\n\tTimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0b\x43ontainerID\x18\x04 \x01(\t\x12\x0e\n\x06HostID\x18\x05 \x01(\t\x12,\n\x08\x42ootTime\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x43ost\x18\x07 \x01(\x02\x12-\n\tStartTime\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06Status\x18\t \x01(\x05\x12+\n\x07\x45ndTime\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08Platform\x18\x0b \x01(\t\x12\x0e\n\x06Region\x18\x0c \x01(\t\x12\x0f\n\x07Runtime\x18\r \x01(\t\x12\x0e\n\x06Memory\x18\x0e \x01(\x05\x12\x33\n\x10\x45xecutionLatency\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\x12\'\n\x03\x45nv\x18\x10 \x03(\x0b\x32\x1a.factclient.Trace.EnvEntry\x12)\n\x04Tags\x18\x11 \x03(\x0b\x32\x1b.factclient.Trace.TagsEntry\x12)\n\x04Logs\x18\x12 \x03(\x0b\x32\x1b.factclient.Trace.LogsEntry\x12\x0c\n\x04\x41rgs\x18\x13 \x03(\t\x1a*\n\x08\x45nvEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tLogsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x04\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,])




_TRACE_ENVENTRY = _descriptor.Descriptor(
  name='EnvEntry',
  full_name='factclient.Trace.EnvEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='factclient.Trace.EnvEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='factclient.Trace.EnvEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=644,
  serialized_end=686,
)

_TRACE_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='factclient.Trace.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='factclient.Trace.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='factclient.Trace.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=688,
  serialized_end=731,
)

_TRACE_LOGSENTRY = _descriptor.Descriptor(
  name='LogsEntry',
  full_name='factclient.Trace.LogsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='factclient.Trace.LogsEntry.key', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='factclient.Trace.LogsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=776,
)

_TRACE = _descriptor.Descriptor(
  name='Trace',
  full_name='factclient.Trace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ID', full_name='factclient.Trace.ID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ChildOf', full_name='factclient.Trace.ChildOf', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Timestamp', full_name='factclient.Trace.Timestamp', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ContainerID', full_name='factclient.Trace.ContainerID', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='HostID', full_name='factclient.Trace.HostID', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='BootTime', full_name='factclient.Trace.BootTime', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Cost', full_name='factclient.Trace.Cost', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='StartTime', full_name='factclient.Trace.StartTime', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Status', full_name='factclient.Trace.Status', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='EndTime', full_name='factclient.Trace.EndTime', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Platform', full_name='factclient.Trace.Platform', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Region', full_name='factclient.Trace.Region', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Runtime', full_name='factclient.Trace.Runtime', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Memory', full_name='factclient.Trace.Memory', index=13,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ExecutionLatency', full_name='factclient.Trace.ExecutionLatency', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Env', full_name='factclient.Trace.Env', index=15,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Tags', full_name='factclient.Trace.Tags', index=16,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Logs', full_name='factclient.Trace.Logs', index=17,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Args', full_name='factclient.Trace.Args', index=18,
      number=19, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TRACE_ENVENTRY, _TRACE_TAGSENTRY, _TRACE_LOGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=93,
  serialized_end=776,
)

_TRACE_ENVENTRY.containing_type = _TRACE
_TRACE_TAGSENTRY.containing_type = _TRACE
_TRACE_LOGSENTRY.containing_type = _TRACE
_TRACE.fields_by_name['Timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRACE.fields_by_name['BootTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRACE.fields_by_name['StartTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRACE.fields_by_name['EndTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRACE.fields_by_name['ExecutionLatency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TRACE.fields_by_name['Env'].message_type = _TRACE_ENVENTRY
_TRACE.fields_by_name['Tags'].message_type = _TRACE_TAGSENTRY
_TRACE.fields_by_name['Logs'].message_type = _TRACE_LOGSENTRY
DESCRIPTOR.message_types_by_name['Trace'] = _TRACE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Trace = _reflection.GeneratedProtocolMessageType('Trace', (_message.Message,), dict(

  EnvEntry = _reflection.GeneratedProtocolMessageType('EnvEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRACE_ENVENTRY,
    __module__ = 'trace_pb2'
    # @@protoc_insertion_point(class_scope:factclient.Trace.EnvEntry)
    ))
  ,

  TagsEntry = _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRACE_TAGSENTRY,
    __module__ = 'trace_pb2'
    # @@protoc_insertion_point(class_scope:factclient.Trace.TagsEntry)
    ))
  ,

  LogsEntry = _reflection.GeneratedProtocolMessageType('LogsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRACE_LOGSENTRY,
    __module__ = 'trace_pb2'
    # @@protoc_insertion_point(class_scope:factclient.Trace.LogsEntry)
    ))
  ,
  DESCRIPTOR = _TRACE,
  __module__ = 'trace_pb2'
  # @@protoc_insertion_point(class_scope:factclient.Trace)
  ))
_sym_db.RegisterMessage(Trace)
_sym_db.RegisterMessage(Trace.EnvEntry)
_sym_db.RegisterMessage(Trace.TagsEntry)
_sym_db.RegisterMessage(Trace.LogsEntry)


_TRACE_ENVENTRY._options = None
_TRACE_TAGSENTRY._options = None
_TRACE_LOGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.duration_pb2 import Duration
from enum import Enum
import trace_pb2 as tracepb
import uuid
import os
import sys
from provider.AWSFact import AWSFact


class Provider(Enum):
    AWS = 0  # AWS Lambda
    ICF = 1  # IBM Cloud Functions
    GCF = 2  # Google Cloud Functions
    ACF = 3  # Azure Cloud Functions
    OWk = 4  # OpenWhisk
    Dok = 5  # Docker
    UND = 6  # UNDEFINED


class Fact:
    _ContainerID = uuid.uuid4()
    _RuntimeString = "{} {} Python {}".format(os.uname()[0], os.uname()[2], sys.version.split(' ')[0])
    config = {}
    _trace = tracepb.Trace()
    _provider = None
    start_time = 0
    base = None

    PHASE = ["provisioned", "start", "update", "done"]

    @staticmethod
    def readFile(path):
        return

    @staticmethod
    def fingerprint(trace, context):
        aws_key = os.getenv("AWS_LAMBDA_LOG_STREAM_NAME")
        gcf_key = os.getenv("X_GOOGLE_FUNCTION_NAME")
        ow_key = os.getenv("__OW_ACTION_NAME")
        acf_key = os.getenv("WEBSITE_HOSTNAME")

        if aws_key:
            AWSFact.get_instances().init(trace, context)
            Fact._provider = Provider.AWS
        elif gcf_key:
            Fact._provider = Provider.GCF
        elif acf_key:
            Fact._provider = Provider.ACF
        elif ow_key:
            if os.path.isfile("/sys/hypervisor/uuid"):
                Fact._provider = Provider.ICF
            else:
                Fact._provider = Provider.OWk

        else:
            Fact._provider = Provider.UND

        return

    @staticmethod
    def boot(configuration):
        Fact.config = configuration
        trace = tracepb.Trace()
        Fact._trace = trace
        now = datetime.now()
        print()
        timestamp = Timestamp()
        timestamp.FromDatetime(now)
        if "inlcudeEnvironment" in configuration and configuration["inlcudeEnvironment"]:
            trace.Env.update(os.environ)
        trace.BootTime.CopyFrom(timestamp)
        trace.ContainerID = str(Fact._ContainerID)
        trace.Runtime = Fact._RuntimeString
        trace.Timestamp.CopyFrom(Fact._trace.BootTime)
        if "lazy_loading" not in configuration or not configuration["lazy_loading"]:
            Fact.load(None)
            if "send_on_update" in configuration and configuration["send_on_update"]:
                Fact.send("provisioned")
        Fact.base = trace

    @staticmethod
    def send(phase):
        if phase not in Fact.PHASE:
            raise ValueError("{} is not a defined phase".format(phase))
        try:
            Fact.config["io"].send(phase, Fact._trace)
        except IOError as e:
            print("failed to send {}:{} - {}\n".format(phase, Fact._trace, e))

    @staticmethod
    def now():
        timestamp = Timestamp()
        timestamp.FromDatetime(datetime.now())
        return timestamp

    @staticmethod
    def load(context):
        Fact.fingerprint(Fact._trace, context)
        Fact.config["io"].connect(os.environ)

    @staticmethod
    def start(context, event):
        trace = tracepb.Trace()
        trace.MergeFrom(Fact.base)
        Fact._trace = trace
        Fact._trace.StartTime.CopyFrom(Fact.now())
        trace.ID = str(uuid.uuid4())
        if Fact._provider is None:
            Fact.load(context)
        assert Fact._provider is not None

        if Fact._provider is Provider.AWS:
            AWSFact.getInstances().collect(Fact._trace, context)

        if "send_on_update" in Fact.config and Fact.config["send_on_update"]:
            Fact.send("start")

    @staticmethod
    def update(context, message, tags):

        assert Fact._provider is not None
        assert Fact.config["io"].connected
        key = int(datetime.now().timestamp() * 1000)
        Fact._trace.Logs[key] = message
        Fact._trace.Logs.update(tags)

        if Fact._provider is Provider.AWS:
            AWSFact.getInstances().collect(Fact._trace, context)

        if "send_on_update" in Fact.config and Fact.config["send_on_update"]:
            Fact.send("update")

    @staticmethod
    def done(context, message, args):
        assert Fact._provider is not None
        assert Fact.config["io"].connected

        Fact._trace.EndTime.CopyFrom(Fact.now())
        key = int(datetime.now().timestamp() * 1000)
        Fact._trace.Logs[key] = message
        Fact._trace.Args.extend(args)
        duration = Duration()
        exec_time = Fact._trace.EndTime.seconds - Fact._trace.StartTime.seconds
        duration.FromSeconds(exec_time)
        Fact._trace.ExecutionLatency.CopyFrom(duration)

        if Fact._provider is Provider.AWS:
            AWSFact.getInstances().collect(Fact._trace, context)

        if "send_on_update" in Fact.config and Fact.config["send_on_update"]:
            Fact.send("done")



from fact import Fact
from datetime import datetime
import trace_pb2

from  factclient.io.TCPSender import TCPSender

io = TCPSender()
Fact.boot({"inlcudeEnvironment": False, "io": io, "send_on_update": True})
Fact.start(None, None)
Fact.update(None, "test updating", {1: "no no no"})
Fact.done(None, "test done", ["jo", "args"])




import socket
from datetime import datetime


class TCPSender:
    address = ""
    port = 9999
    connected = False

    def connect(self, env):
        self.address = env["fact_tcp_address"]
        if "fact_tcp_port" in env:
            self.port = env["fact_tcp_port"]
        self.connected = True

    def send(self, message, trace):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.connect((self.address, self.port))
            key = int(datetime.now().timestamp() * 1000)
            trace.Logs[key] = message
            ser_trace = trace.SerializeToString()
            print("address: {} port: {}".format(self.address,self.port))
            s.sendall(ser_trace)
            s.close()



from datetime import datetime


class ConsoleLogging:
    connected = True

    def connect(self, env):
        return

    def send(self, message, trace):
        print("[{}] [{}] {}".format(datetime.now().timestamp() * 1000, message, trace))
